/**
 * gclc-socket:net.bigeon.gclc.socket.ConnexionManager.java
 * Created on: Nov 18, 2017
 */
package net.bigeon.gclc.socket;

/*-
 * #%L
 * GCLC Socket
 * %%
 * Copyright (C) 2016 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.util.Collection;

/** A manager for connected elements.
 * <p>
 * Connected elements are given an identifier (unique) at connexion.
 *
 * @author Emmanuel Bigeon
 * @param <T> the type of object connected */
public interface ConnexionManager<T> {
    /** Add a connection in the name.
     *
     * @param handle the connected object
     * @return the name */
    String addConnexion(T handle);

    /** Disconnect an element.
     *
     * @param id the element connection id
     * @return the object being disconnected */
    T disconnect(String id);

    /** Get the connected object.
     *
     * @param id the connexion id
     * @return the object */
    T get(String id);

    /** Get the connected elements' ids.
     *
     * @return the connected elements' ids */
    Collection<String> getConnected();

    /** Test if a connection is active.
     *
     * @param id the connexion id
     * @return if the connection is active. */
    boolean isConnected(String id);

    /** Add a lock on the disconnection.
     * <p>
     * This lock will <strong>not</strong> prevent calls to
     * {@link #disconnect(String)}. It will however stop them from completing after
     * the effective disconnection of the specified connection.
     * <p>
     * Calls to {@link #releaseDisconnexionLock(String)} remove a lock (at a pace of
     * one for one).
     *
     * @param id the connexion id */
    void lockDisconnexion(String id);

    /** Release one lock on a disconnection
     *
     * @param id the connexion being released. */
    void releaseDisconnexionLock(String id);

    /** Wait for calls to {@link #disconnect(String)}
     *
     * @param id the connexion id
     * @throws InterruptedException if the wait was interrupted. */
    void waitDisconnexion(String id) throws InterruptedException;
}
