/**
 * 
 */
package net.bigeon.gclc.swt;

import static org.mockito.ArgumentMatchers.any;
/*-
 * #%L
 * GCLC swt
 * %%
 * Copyright (C) 2015 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import static org.mockito.Mockito.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.concurrent.atomic.AtomicInteger;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.junit.Test;
import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

import net.bigeon.gclc.swt.io.ConsolePromptManager;

/** @author Emmanuel Bigeon */
public class ConsolePromptManagerTest {

    private final Label label = mock(Label.class);

    private final ConsolePromptManager manager = new ConsolePromptManager(label);

    /** Test method for
     * {@link net.bigeon.gclc.swt.io.ConsolePromptManager#setPrompt(java.lang.String)}. */
    @Test
    public void testSetPrompt() {
        manager.setPrompt("abc");
        verify(label).setText("abc");
    }

    /** Test method for
     * {@link net.bigeon.gclc.swt.io.ConsolePromptManager#setStream(java.io.BufferedReader)}.
     * 
     * @throws IOException if error */
    @Test
    public void testSetStream() throws IOException {
        // Create the dispaly, in case...
        final Display d = mock(Display.class);
        when(label.getDisplay()).thenReturn(d);
        Mockito.doAnswer(invocation -> {
            ((Runnable) invocation.getArgument(0)).run();
            return null;
        }).when(d).asyncExec(any(Runnable.class));
        Mockito.doAnswer(invocation -> {
            ((Runnable) invocation.getArgument(0)).run();
            return null;
        }).when(d).syncExec(any(Runnable.class));
        // Test.
        BufferedReader output = mock(BufferedReader.class);
        final Object lock = new Object();
        final AtomicInteger calls = new AtomicInteger(0);
        Mockito.when(output.readLine()).then(new Answer<String>() {
            String[] ans = { "A line" };

            @Override
            public String answer(final InvocationOnMock invocation) throws Throwable {
                synchronized (lock) {
                    if (calls.get() >= ans.length) {
                        calls.incrementAndGet();
                        lock.notify();
                        return null;
                    }
                    lock.notify();
                    return ans[calls.getAndIncrement()];
                }
            }
        });
        manager.setStream(output);
        manager.setStream(output);
        synchronized (lock) {
            while (calls.get() < 2) {
                try {
                    lock.wait(10);
                } catch (final InterruptedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
        verify(output, times(2)).readLine();
        output = mock(BufferedReader.class);
        calls.set(0);
        Mockito.when(output.readLine()).then(new Answer<String>() {
            String[] ans = { "A line" };

            @Override
            public String answer(final InvocationOnMock invocation) throws Throwable {
                synchronized (lock) {
                    if (calls.get() >= ans.length) {
                        calls.incrementAndGet();
                        lock.notify();
                        return null;
                    }
                    lock.notify();
                    return ans[calls.getAndIncrement()];
                }
            }
        });
        manager.setStream(output);
        while (calls.get() < 2) {
            synchronized (lock) {
                try {
                    lock.wait(10);
                } catch (final InterruptedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
        manager.setStream(null);
        manager.setStream(output);
        while (calls.get() < 3) {
            synchronized (lock) {
                try {
                    lock.wait(10);
                } catch (final InterruptedException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
        verify(output, times(3)).readLine();
        verify(label, times(2)).setText("A line");
    }
}
