/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.GCLCConstantsTest.java
 * Created on: Jun 8, 2016
 */
package net.bigeon.gclc;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.List;

import org.junit.Test;

import net.bigeon.gclc.exception.CommandParsingException;

/** Test class for {@link GCLCConstants}
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings({ "nls", "static-method" })
public class GCLCConstantsTest {

    @Test
    public void testSplitCommandSingle() throws CommandParsingException {
        List<String> res;
        res = GCLCConstants.splitCommand("aCommand");
        assertEquals("single word command should have one element", 1, res.size());
        assertEquals("Command should be preserved", "aCommand", res.get(0));
    }

    @Test
    public void testSplitCommandSeveral() throws CommandParsingException {
        List<String> res;
        res = GCLCConstants.splitCommand("aCommand with some arguments");
        assertEquals("Command size", 4, res.size());
        assertEquals("Elements should be preserved", "aCommand", res.get(0));
        assertEquals("Command should be preserved", "with", res.get(1));
        assertEquals("Command should be preserved", "some", res.get(2));
        assertEquals("Command should be preserved", "arguments", res.get(3));
    }

    /** Test method for
     * {@link net.bigeon.gclc.GCLCConstants#splitCommand(java.lang.String)}.
     *
     * @throws CommandParsingException if an error occured */
    @Test
    public void testSplitCommandMultiSpaces() throws CommandParsingException {
        List<String> res;
        res = GCLCConstants.splitCommand("aCommand with  some arguments");
        assertEquals("Command size", 4, res.size());
        assertEquals("Command should be preserved", "aCommand", res.get(0));
        assertEquals("Command should be preserved", "with", res.get(1));
        assertEquals("Command should be preserved", "some", res.get(2));
        assertEquals("Command should be preserved", "arguments", res.get(3));
    }

    public void testSplitCommandWithString() throws CommandParsingException {
        List<String> res;
        res = GCLCConstants.splitCommand("aCommand \"with some\" arguments");
        assertEquals("Command size", 3, res.size());
        assertEquals("Command should be preserved", "aCommand", res.get(0));
        assertEquals("Command should be preserved", "with some", res.get(1));
        assertEquals("Command should be preserved", "arguments", res.get(2));

        res = GCLCConstants.splitCommand("aCommand with \"some arguments\"");
        assertEquals("Command size", 3, res.size());
        assertEquals("Command should be preserved", "aCommand", res.get(0));
        assertEquals("Command should be preserved", "with", res.get(1));
        assertEquals("Command should be preserved", "some arguments", res.get(2));
    }

    public void testSplitCommandWithEscape() throws CommandParsingException {
        List<String> res;
        res = GCLCConstants.splitCommand("aCommand with\\ some arguments");
        assertEquals("Command size", 3, res.size());
        assertEquals("Command should be preserved", "aCommand", res.get(0));
        assertEquals("Command should be preserved", "with some", res.get(1));
        assertEquals("Command should be preserved", "arguments", res.get(2));
    }

    public void testSplitCommandWithQuoteInString() throws CommandParsingException {
        List<String> res;
        res = GCLCConstants.splitCommand("aCommand wi\\\"th some arguments");
        assertEquals("Command size", 4, res.size());
        assertEquals("Command should be preserved", "aCommand", res.get(0));
        assertEquals("Command should be preserved", "wi\"th", res.get(1));
        assertEquals("Command should be preserved", "some", res.get(2));
        assertEquals("Command should be preserved", "arguments", res.get(3));

    }

    public void testSplitCommandInvalidQuoting() throws CommandParsingException {
        List<String> res;
        try {
            // Wrong lines?
            res = GCLCConstants.splitCommand("aCommand with \"some ar\"guments");
            fail("Misplaced quotes should fail");
        } catch (final CommandParsingException e) {
            // ok
        }
    }

}
